import {
  Inject,
  Injectable,
  ForbiddenException,
  UnauthorizedException,
} from '@nestjs/common'
import { PasswordService } from './password.service'
import { PrismaService } from 'nestjs-prisma'
import { Token, TokenPayload } from './dto/token.dto'
import { JwtService } from '@nestjs/jwt'
import { securityConfig, SecurityConfig } from 'src/common/configs'
import { CreateUserDto } from 'src/users/dto/create-user.dto'
import { EmailSendDto, EmailScene } from 'src/email/dto/email.dto'
import { EmailService } from 'src/email/email.service'

@Injectable()
export class AuthService {
  constructor(
    private passwordService: PasswordService,
    private jwtService: JwtService,
    private prismaService: PrismaService,
    private emailService: EmailService,
    @Inject(securityConfig.KEY)
    private secureConfig: SecurityConfig,
  ) {}

  async register(userToCreate: CreateUserDto) {
    const tokenPayload = this.jwtService.verify<EmailSendDto>(
      userToCreate.token,
      {
        secret: this.emailService.getEmailJwtSecret(
          userToCreate.verificationCode,
          EmailScene.register,
        ),
      },
    )
    if (
      tokenPayload.email !== userToCreate.email ||
      tokenPayload.scene !== 'register'
    ) {
      throw new ForbiddenException('请输入正确的邮箱')
    }

    const hashedPassword = await this.passwordService.hashPassword(
      userToCreate.password,
    )
    const user = await this.prismaService.user.create({
      data: {
        username: userToCreate.username,
        email: userToCreate.email,
        password: hashedPassword,
      },
    })
    return this.generateTokens({ userId: user.id })
  }

  async login(email: string, password: string) {
    const user = await this.prismaService.user.findUniqueOrThrow({
      where: { email },
    })

    const passwordValid = await this.passwordService.validatePassword(
      password,
      user.password,
    )

    if (!passwordValid) {
      throw new ForbiddenException('Invalid password')
    }

    return this.generateTokens({ userId: user.id })
  }

  async refreshToken(token: string) {
    try {
      const { userId } = this.jwtService.verify<TokenPayload>(token, {
        secret: this.secureConfig.jwt_refresh_secret,
      })
      return this.generateTokens({ userId })
    } catch (e) {
      console.error(e)
      throw new UnauthorizedException(e.message)
    }
  }

  private generateTokens(payload: TokenPayload): Token {
    const accessToken = this.jwtService.sign(payload, {
      secret: this.secureConfig.jwt_access_secret,
      expiresIn: this.secureConfig.expiresIn,
    })
    const refreshToken = this.jwtService.sign(payload, {
      secret: this.secureConfig.jwt_refresh_secret,
      expiresIn: this.secureConfig.refreshIn,
    })

    return { accessToken, refreshToken }
  }
}
